﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace SemiColinGames {
  class Scene {
    public bool Enabled = false;

    Color backgroundColor = Color.CornflowerBlue;

    readonly GraphicsDevice graphics;
    readonly Camera camera;

    readonly RenderTarget2D sceneTarget;
    readonly RenderTarget2D lightingTarget;
    readonly BasicEffect lightingEffect;

    readonly SpriteBatch spriteBatch;

    public Scene(GraphicsDevice graphics, Camera camera) {
      this.graphics = graphics;
      this.camera = camera;

      sceneTarget = new RenderTarget2D(
        graphics, camera.Width, camera.Height, false /* mipmap */,
        graphics.PresentationParameters.BackBufferFormat, DepthFormat.Depth24);
      lightingTarget = new RenderTarget2D(
        graphics, camera.Width, camera.Height, false /* mipmap */,
        graphics.PresentationParameters.BackBufferFormat, DepthFormat.Depth24);

      lightingEffect = new BasicEffect(graphics) {
        World = Matrix.CreateTranslation(0, 0, 0),
        View = Matrix.CreateLookAt(Vector3.Backward, Vector3.Zero, Vector3.Up),
        VertexColorEnabled = true
      };

      spriteBatch = new SpriteBatch(graphics);
    }

    public void Draw(World world, Player player, LinesOfSight linesOfSight) {
      graphics.SetRenderTarget(null);
      graphics.Clear(backgroundColor);
      if (!Enabled) {
        return;
      }

      // Draw scene to sceneTarget.
      graphics.SetRenderTarget(sceneTarget);
      graphics.Clear(backgroundColor);

      // Draw background.
      spriteBatch.Begin(SpriteSortMode.Deferred, null, SamplerState.LinearWrap, null, null);
      Rectangle bgTarget = new Rectangle(0, 0, camera.Width, camera.Height);

      float xScale = 1f / 16;
      for (int i = 0; i < Textures.Backgrounds.Length; i++) {
        int yOffset = Textures.Backgrounds[i].Height - camera.Height - 24;
        Rectangle bgSource = new Rectangle(
          (int) (camera.Left * xScale), yOffset, camera.Width, camera.Height);
        spriteBatch.Draw(Textures.Backgrounds[i], bgTarget, bgSource, Color.White);
        xScale *= 2;
      }
      spriteBatch.End();

      // Set up transformation matrix for drawing world objects.
      Matrix transform = Matrix.CreateTranslation(-camera.Left, -camera.Top, 0);
      spriteBatch.Begin(
          SpriteSortMode.Deferred, null, SamplerState.LinearWrap, null, null, null, transform);

      // Draw player.
      player.Draw(spriteBatch);

      // Draw foreground tiles.
      world.Draw(spriteBatch);

      // Aaaaand we're done.
      spriteBatch.End();

      // Draw lighting to lightingTarget.
      graphics.SetRenderTarget(lightingTarget);
      graphics.Clear(new Color(0, 0, 0, 0f));
      lightingEffect.Projection = camera.Projection;
      linesOfSight.Draw(player, world.CollisionTargets, graphics, lightingEffect);

      // Draw debug rects & lines on top.
      Debug.Draw(graphics, lightingEffect);

      // Draw sceneTarget to screen.
      graphics.SetRenderTarget(null);
      spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend,
          SamplerState.PointClamp, DepthStencilState.Default,
          RasterizerState.CullNone);
      Rectangle drawRect = new Rectangle(
        0, 0, graphics.Viewport.Width, graphics.Viewport.Height);
      spriteBatch.Draw(sceneTarget, drawRect, Color.White);
      spriteBatch.Draw(lightingTarget, drawRect, Color.White);

      // Draw debug toasts.
      Debug.DrawToasts(spriteBatch);

      spriteBatch.End();
    }
  }
}
